@@ -21,6 +21,7 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
         o = random.choice(os)
         f = random.choice(firevers)
         return "Mozilla/5.0 ({0}; Win64; x64; rv:{1}) Gecko/20100101 Firefox/{1}".format(o,f)
+    print("# {}".format(title),file=sys.stderr,flush=True)
     ss = requests.Session()
     tgtdir = os.path.expanduser(tgtdir)
     if not os.path.isdir(tgtdir) :
@@ -58,20 +59,26 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
     if debug :
         print(rtext)
     chapters = list()
-    for m in re.findall(r"(<dd>|<li>)\s*<a\s+href=\"(\S*?\d+\.(htm|html))\"\s*>\s*(.*?)\s*</a>",rtext,re.DOTALL) :
+    for m in re.findall(r"(<dd>|<li>)\s*<a\s+href=\"(\S*?\d+\.(htm|html))\".*?>\s*(.*?)\s*</a>",rtext,re.DOTALL) :
         url=m[1].split("/")[-1]
+        m2 = re.search(r"(\d+)",url)
+        if m2 :
+            if len(m2.group(1)) < 4 :
+                continue
+        else :
+            continue
         chapter=m[3]
         chapters.append((url,chapter))
     def cid(c) :
         s = c[0]
         if not s :
-            return 2**31
+            return 2**60
         s = re.sub(r"(\.html|\.htm)",'',s)
         m = re.search(r"(\d+)",s)
         if m :
-            return int(m.group(1))
+            return 2**(30+len(m.group(1)))+int(m.group(1))
         else :
-            return 2**31
+            return 2**60
     chapters = sorted(list(set(chapters)),key=cid)
     if debug :
         print(chapters)
@@ -85,6 +92,8 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
         # redownload the last chapter to fix the prev/next link
         if cname and cname < last :
             continue
+        if page.endswith(".htm") or page.endswith(".html") :
+            page = "".join(page.split("/")[:-1])
         if not url.startswith("http") :
             url = page.rstrip("/") + "/" + url
         chapter=m[1]
@@ -95,16 +104,24 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
                 print("# {}".format(url),file=sys.stderr,flush=True)
                 print("# {}".format(user_agent),file=sys.stderr,flush=True)
             rsp = ss.get(url,headers=user_agent,timeout=20,verify=False)
+            rsp.encoding = rsp.apparent_encoding
         except :
            print("# {}".format(traceback.format_exc().splitlines()[-1]),file=sys.stderr,flush=True)
            break
-        rsp.encoding = rsp.apparent_encoding
+        if debug :
+            print(rsp.text)
         m = re.search(r"<div (class|id)=\"content\".*?>(.*?)</div>",rsp.text,re.DOTALL)
         context = ""
+        divgood = True
         if m :
-            context = m.group(2).replace("<br/>","</br>\n")
-            context = m.group(2).replace("<br/>","<br/>\n")
-        else :
+            if debug :
+                print("# matched to div class/id=content", file=sys.stderr, flush=True)
+            context = m.group(2)
+            if len(context) < 1500 :
+                divgood = False
+        if not m or not divgood :
+            if debug :
+                print("# wild match", file=sys.stderr, flush=True)
             lines = [ln for ln in rsp.text.splitlines()]
             started = False
             for _, ln in enumerate(lines) :
@@ -118,10 +135,11 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
         if len(context) < 200 :
             print("# context length abnormal. most likely hit with rate limiter", file=sys.stderr, flush=True)
             break
-        if not re.search(r"<br/>\s*</br>",context,re.DOTALL) :
-            context = context.replace("<br/>","</br></br>")
+        context = re.sub(r"</br>","<br/>\n",context)
+        context = re.sub(r"<br\s*/>","<br/>\n",context)
         if not re.search(r"<br/>\s*<br/>",context,re.DOTALL) :
             context = context.replace("<br/>","<br/><br/>")
+        context = context.replace("<br/>","<br/>\n")
         if ix == 0 :
             prevpg = "index.html"
             cdone += 1 
@@ -176,7 +194,7 @@ def adhoc_fetch(title,page,limit=10,tgtdir=os.getcwd(),debug=False) :
         cdone+=1
         time.sleep(float(random.choice([i for i in range(30)])/10.0))
     report = report.rstrip(",")
-    print("# updates : {}".format(report), file=sys.stderr, flush=True)
+    #print("# updates : {}".format(report), file=sys.stderr, flush=True)
     return report
 
 
